@@ -74,7 +74,6 @@ struct App
   GtkWidget      *show_icon_checkbox;
 
   /* We store the event timestamp when the Apply button is clicked */
-  GtkWidget      *apply_button;
   guint32         apply_button_clicked_timestamp;
 
   GtkWidget      *area;
@@ -85,11 +84,6 @@ struct App
   DBusGConnection *connection;
   DBusGProxy *proxy;
   DBusGProxyCall *proxy_call;
-
-  enum {
-    APPLYING_VERSION_1,
-    APPLYING_VERSION_2
-  } apply_configuration_state;
 };
 
 static void rebuild_gui (App *app);
@@ -257,7 +251,6 @@ add_key (GtkWidget *widget,
   GtkComboBox *box = GTK_COMBO_BOX (widget);
   GtkTreeModel *model = gtk_combo_box_get_model (box);
   GtkListStore *store = GTK_LIST_STORE (model);
-  gboolean retval;
 
   info.text = text;
   info.found = FALSE;
@@ -276,11 +269,6 @@ add_key (GtkWidget *widget,
                                          5, rotation,
                                          -1);
 
-      retval = TRUE;
-    }
-  else
-    {
-      retval = FALSE;
     }
 }
 
@@ -544,13 +532,11 @@ rebuild_on_off_radios (App *app)
 {
   gboolean sensitive;
   gboolean on_active;
-  gboolean off_active;
 
   g_signal_handlers_block_by_func (app->monitor_switch, G_CALLBACK (monitor_switch_active_cb), app);
 
   sensitive = FALSE;
   on_active = FALSE;
-  off_active = FALSE;
 
   if (!gnome_rr_config_get_clone (app->current_configuration) && app->current_output)
     {
@@ -560,7 +546,6 @@ rebuild_on_off_radios (App *app)
         sensitive = FALSE;
 
       on_active = gnome_rr_output_info_is_active (app->current_output);
-      off_active = !on_active;
     }
 
   gtk_widget_set_sensitive (app->monitor_switch, sensitive);
@@ -691,8 +676,6 @@ rebuild_resolution_combo (App *app)
 static void
 rebuild_gui (App *app)
 {
-  gboolean sensitive;
-
   /* We would break spectacularly if we recursed, so
    * just assert if that happens
    */
@@ -700,22 +683,12 @@ rebuild_gui (App *app)
 
   app->ignore_gui_changes = TRUE;
 
-  sensitive = app->current_output ? TRUE : FALSE;
-
-#if 0
-  g_debug ("rebuild gui, is on: %d", app->current_output->on);
-#endif
-
   rebuild_mirror_screens (app);
   rebuild_current_monitor_label (app);
   rebuild_on_off_radios (app);
   rebuild_resolution_combo (app);
   rebuild_rotation_combo (app);
 
-#if 0
-  g_debug ("sensitive: %d, on: %d", sensitive, gnome_rr_output_info_is_active (app->current_output));
-#endif
-
   app->ignore_gui_changes = FALSE;
 }
 
@@ -1925,7 +1898,6 @@ paint_output (App *app, cairo_t *cr, int i)
 
   cairo_save (cr);
 
-  layout = get_display_name (app, output);
   layout_set_font (layout, "Sans 10");
   pango_layout_get_pixel_extents (layout, &ink_extent, &log_extent);
 
@@ -2013,7 +1985,6 @@ on_area_paint (FooScrollArea  *area,
                gpointer        data)
 {
   App *app = data;
-  double scale;
   GList *connected_outputs = NULL;
   GList *list;
 
@@ -2022,13 +1993,8 @@ on_area_paint (FooScrollArea  *area,
   if (!app->current_configuration)
     return;
 
-  scale = compute_scale (app);
   connected_outputs = list_connected_outputs (app, NULL, NULL);
 
-#if 0
-  g_debug ("scale: %f", scale);
-#endif
-
   for (list = connected_outputs; list != NULL; list = list->next)
     {
       paint_output (app, cr, g_list_position (connected_outputs, list));
@@ -2138,7 +2104,6 @@ begin_version2_apply_configuration (App *app, GdkWindow *parent_window, guint32
                                           "org.gnome.SettingsDaemon.XRANDR_2");
   g_assert (app->proxy != NULL); /* that call does not fail unless we pass bogus names */
 
-  app->apply_configuration_state = APPLYING_VERSION_2;
   app->proxy_call = dbus_g_proxy_begin_call (app->proxy, "ApplyConfiguration",
                                              apply_configuration_returned_cb, app,
                                              NULL,
@@ -2152,27 +2117,6 @@ begin_version2_apply_configuration (App *app, GdkWindow *parent_window, guint32
    */
 }
 
-static void
-begin_version1_apply_configuration (App *app)
-{
-  app->proxy = dbus_g_proxy_new_for_name (app->connection,
-                                          "org.gnome.SettingsDaemon",
-                                          "/org/gnome/SettingsDaemon/XRANDR",
-                                          "org.gnome.SettingsDaemon.XRANDR");
-  g_assert (app->proxy != NULL); /* that call does not fail unless we pass bogus names */
-
-  app->apply_configuration_state = APPLYING_VERSION_1;
-  app->proxy_call = dbus_g_proxy_begin_call (app->proxy, "ApplyConfiguration",
-                                             apply_configuration_returned_cb, app,
-                                             NULL,
-                                             G_TYPE_INVALID,
-                                             G_TYPE_INVALID);
-  /* FIXME: we don't check for app->proxy_call == NULL, which could happen if
-   * the connection was disconnected.  This is left as an exercise for the
-   * reader.
-   */
-}
-
 static void
 ensure_current_configuration_is_saved (void)
 {
@@ -2215,21 +2159,10 @@ apply_configuration_returned_cb (DBusGProxy       *proxy,
   success = dbus_g_proxy_end_call (proxy, call_id, &error, G_TYPE_INVALID);
 
   if (!success) {
-    if (app->apply_configuration_state == APPLYING_VERSION_2
-        && g_error_matches (error, DBUS_GERROR, DBUS_GERROR_UNKNOWN_METHOD)) {
-      g_error_free (error);
-
-      g_object_unref (app->proxy);
-      app->proxy = NULL;
-
-      begin_version1_apply_configuration (app);
-      return;
-    } else {
-      /* We don't pop up an error message; gnome-settings-daemon already does that
-       * in case the selected RANDR configuration could not be applied.
-       */
-      g_error_free (error);
-    }
+    /* We don't pop up an error message; gnome-settings-daemon already does that
+     * in case the selected RANDR configuration could not be applied.
+     */
+    g_error_free (error);
   }
 
   g_object_unref (app->proxy);
@@ -2273,6 +2206,8 @@ apply (App *app)
   GError *error = NULL;
   GdkWindow *window;
 
+  app->apply_button_clicked_timestamp = gtk_get_current_event_time ();
+
   if (!sanitize_and_save_configuration (app))
     return;
 
@@ -2471,19 +2406,6 @@ dialog_map_event_cb (GtkWidget *widget, GdkEventAny *event, gpointer data)
 }
 
 
-static void
-apply_button_clicked_cb (GtkButton *button, gpointer data)
-{
-  App *app = data;
-
-  /* We simply store the timestamp at which the Apply button was clicked.
-   * We'll just wait for the dialog to return from gtk_dialog_run(), and
-   * *then* use the timestamp when applying the RANDR configuration.
-   */
-
-  app->apply_button_clicked_timestamp = gtk_get_current_event_time ();
-}
-
 static GtkWidget*
 _gtk_builder_get_widget (GtkBuilder *builder, const gchar *name)
 {
@@ -2600,10 +2522,6 @@ run_application (void)
 
   gtk_container_add (GTK_CONTAINER (align), app->area);
 
-  app->apply_button = _gtk_builder_get_widget (builder, "apply_button");
-  g_signal_connect (app->apply_button, "clicked",
-                    G_CALLBACK (apply_button_clicked_cb), app);
-
   on_screen_changed (app->screen, app);
 
   g_signal_connect_swapped (_gtk_builder_get_widget (builder, "apply_button"),
