@@ -24,12 +24,13 @@
 
 #include <glib-object.h>
 #include <NetworkManager.h>
+#include <nm-device.h>
 
 G_BEGIN_DECLS
 
-const gchar     *panel_device_type_to_icon_name                 (NMDeviceType type);
-const gchar     *panel_device_type_to_localized_string          (NMDeviceType type);
-const gchar     *panel_device_type_to_sortable_string           (NMDeviceType type);
+const gchar     *panel_device_to_icon_name                 (NMDevice *device);
+const gchar     *panel_device_to_localized_string          (NMDevice *device);
+const gchar     *panel_device_to_sortable_string           (NMDevice *device);
 const gchar     *panel_ap_mode_to_localized_string              (NM80211Mode mode);
 const gchar     *panel_device_state_to_localized_string         (NMDeviceState type);
 
