@@ -61,7 +61,9 @@ struct _GnomeControlCenterPrivate
   GtkWidget  *window;
   GtkWidget  *search_entry;
 
+  GMenuTree  *menu_tree;
   GtkListStore *store;
+  GHashTable *category_views;
 
   GtkTreeModel *search_filter;
   GtkWidget *search_view;
@@ -89,12 +91,8 @@ activate_panel (GnomeControlCenter *shell,
                 const gchar        *icon_name)
 {
   GnomeControlCenterPrivate *priv = shell->priv;
-  GAppInfo *appinfo;
-  GError *err = NULL;
-  GdkAppLaunchContext *ctx;
   GType panel_type = G_TYPE_INVALID;
   GList *panels, *l;
-  GKeyFile *key_file;
 
   /* check if there is an plugin that implements this panel */
   panels = g_io_extension_point_get_extensions (priv->extension_point);
@@ -145,6 +143,7 @@ activate_panel (GnomeControlCenter *shell,
 
           /* set the title of the window */
           gtk_window_set_title (GTK_WINDOW (priv->window), name);
+          gtk_window_set_default_icon_name (icon_name);
           gtk_window_set_icon_name (GTK_WINDOW (priv->window), icon_name);
 
           gtk_widget_show (panel);
@@ -157,51 +156,12 @@ activate_panel (GnomeControlCenter *shell,
           gtk_widget_set_size_request (priv->window, FIXED_WIDTH, -1);
           gtk_widget_get_preferred_height (GTK_WIDGET (priv->window),
                                            NULL, &nat_height);
-          g_debug ("Setting panel height: %d", nat_height);
           gtk_window_resize (GTK_WINDOW (priv->window),
                              FIXED_WIDTH,
                              nat_height);
           return;
         }
     }
-
-
-  /* if a plugin was not found, then start app directly */
-
-  if (err)
-    {
-      g_warning ("Error starting \"%s\": %s", desktop_file, err->message);
-
-      g_error_free (err);
-      err = NULL;
-
-      return;
-    }
-
-  key_file = g_key_file_new ();
-  g_key_file_load_from_file (key_file, desktop_file, 0, &err);
-
-  appinfo = (GAppInfo*) g_desktop_app_info_new_from_keyfile (key_file);
-
-  g_key_file_free (key_file);
-  key_file = NULL;
-
-
-  ctx = gdk_app_launch_context_new ();
-  gdk_app_launch_context_set_screen (ctx, gdk_screen_get_default ());
-  gdk_app_launch_context_set_timestamp (ctx, priv->last_time);
-
-  g_app_info_launch (appinfo, NULL, G_APP_LAUNCH_CONTEXT (ctx), &err);
-
-  g_object_unref (appinfo);
-  g_object_unref (ctx);
-
-  if (err)
-    {
-      g_warning ("Error starting \"%s\": %s", desktop_file, err->message);
-      g_error_free (err);
-      err = NULL;
-    }
 }
 
 static void
@@ -218,10 +178,17 @@ shell_show_overview_page (GnomeControlCenterPrivate *priv)
 
   /* reset window title and icon */
   gtk_window_set_title (GTK_WINDOW (priv->window), priv->default_window_title);
+  gtk_window_set_default_icon_name (priv->default_window_icon);
   gtk_window_set_icon_name (GTK_WINDOW (priv->window),
                             priv->default_window_icon);
 }
 
+void
+gnome_control_center_set_overview_page (GnomeControlCenter *center)
+{
+  shell_show_overview_page (center->priv);
+}
+
 static void
 item_activated_cb (CcShellCategoryView *view,
                    gchar               *name,
@@ -313,7 +280,7 @@ keynav_failed (GtkIconView        *current_view,
         break;
     }
 
-  if (direction == GTK_DIR_DOWN && v->next != NULL)
+  if (direction == GTK_DIR_DOWN && v != NULL && v->next != NULL)
     {
       new_view = v->next->data;
 
@@ -350,7 +317,7 @@ keynav_failed (GtkIconView        *current_view,
       res = TRUE;
     }
 
-  if (direction == GTK_DIR_UP && v->prev != NULL)
+  if (direction == GTK_DIR_UP && v != NULL && v->prev != NULL)
     {
       new_view = v->prev->data;
 
@@ -479,7 +446,7 @@ search_entry_changed_cb (GtkEntry                  *entry,
     {
       shell_show_overview_page (priv);
       g_object_set (G_OBJECT (entry),
-                    "secondary-icon-name", "gtk-find",
+                    "secondary-icon-name", "edit-find-symbolic",
                     "secondary-icon-activatable", FALSE,
                     "secondary-icon-sensitive", FALSE,
                     NULL);
@@ -490,7 +457,7 @@ search_entry_changed_cb (GtkEntry                  *entry,
       gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook),
                                      SEARCH_PAGE);
       g_object_set (G_OBJECT (entry),
-                          "secondary-icon-name", "gtk-clear",
+                          "secondary-icon-name", "edit-clear-symbolic",
                           "secondary-icon-activatable", TRUE,
                           "secondary-icon-sensitive", TRUE,
                           NULL);
@@ -591,32 +558,53 @@ setup_search (GnomeControlCenter *shell)
 }
 
 static void
-fill_model (GnomeControlCenter *shell)
+maybe_add_category_view (GnomeControlCenter *shell,
+                         const char         *name)
+{
+  GtkTreeModel *filter;
+  GtkWidget *categoryview;
+
+  if (g_hash_table_lookup (shell->priv->category_views, name) != NULL)
+    return;
+
+  /* create new category view for this category */
+  filter = gtk_tree_model_filter_new (GTK_TREE_MODEL (shell->priv->store),
+                                      NULL);
+  gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (filter),
+                                          (GtkTreeModelFilterVisibleFunc) category_filter_func,
+                                          g_strdup (name), g_free);
+
+  categoryview = cc_shell_category_view_new (name, filter);
+  gtk_box_pack_start (GTK_BOX (shell->priv->main_vbox), categoryview, FALSE, TRUE, 0);
+
+  g_signal_connect (cc_shell_category_view_get_item_view (CC_SHELL_CATEGORY_VIEW (categoryview)),
+                    "desktop-item-activated",
+                    G_CALLBACK (item_activated_cb), shell);
+
+  gtk_widget_show (categoryview);
+
+  g_signal_connect (cc_shell_category_view_get_item_view (CC_SHELL_CATEGORY_VIEW (categoryview)),
+                    "focus-in-event",
+                    G_CALLBACK (category_focus_in), shell);
+  g_signal_connect (cc_shell_category_view_get_item_view (CC_SHELL_CATEGORY_VIEW (categoryview)),
+                    "focus-out-event",
+                    G_CALLBACK (category_focus_out), shell);
+  g_signal_connect (cc_shell_category_view_get_item_view (CC_SHELL_CATEGORY_VIEW (categoryview)),
+                    "keynav-failed",
+                    G_CALLBACK (keynav_failed), shell);
+
+  g_hash_table_insert (shell->priv->category_views, g_strdup (name), categoryview);
+}
+
+static void
+reload_menu (GnomeControlCenter *shell)
 {
   GSList *list, *l;
   GMenuTreeDirectory *d;
-  GMenuTree *tree;
-
-  GnomeControlCenterPrivate *priv = shell->priv;
-
-  gtk_container_set_border_width (GTK_CONTAINER (shell->priv->main_vbox), 10);
-  gtk_container_set_focus_vadjustment (GTK_CONTAINER (shell->priv->main_vbox),
-                                       gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (shell->priv->scrolled_window)));
-
-  tree = gmenu_tree_lookup (MENUDIR "/gnomecc.menu", 0);
-
-  if (!tree)
-    {
-      g_warning ("Could not find control center menu");
-      return;
-    }
-
-  d = gmenu_tree_get_root_directory (tree);
 
+  d = gmenu_tree_get_root_directory (shell->priv->menu_tree);
   list = gmenu_tree_directory_get_contents (d);
 
-  priv->store = (GtkListStore *) cc_shell_model_new ();
-
   for (l = list; l; l = l->next)
     {
       GMenuTreeItemType type;
@@ -624,44 +612,20 @@ fill_model (GnomeControlCenter *shell)
 
       if (type == GMENU_TREE_ITEM_DIRECTORY)
         {
-          GtkTreeModel *filter;
-          GtkWidget *categoryview;
           GSList *contents, *f;
           const gchar *dir_name;
 
           contents = gmenu_tree_directory_get_contents (l->data);
           dir_name = gmenu_tree_directory_get_name (l->data);
 
-          /* create new category view for this category */
-          filter = gtk_tree_model_filter_new (GTK_TREE_MODEL (priv->store),
-                                              NULL);
-          gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (filter),
-                                                  (GtkTreeModelFilterVisibleFunc) category_filter_func,
-                                                  g_strdup (dir_name), g_free);
-
-          categoryview = cc_shell_category_view_new (dir_name, filter);
-          gtk_box_pack_start (GTK_BOX (shell->priv->main_vbox), categoryview, FALSE, TRUE, 0);
-          g_signal_connect (cc_shell_category_view_get_item_view (CC_SHELL_CATEGORY_VIEW (categoryview)),
-                            "desktop-item-activated",
-                            G_CALLBACK (item_activated_cb), shell);
-          gtk_widget_show (categoryview);
-          g_signal_connect (cc_shell_category_view_get_item_view (CC_SHELL_CATEGORY_VIEW (categoryview)),
-                           "focus-in-event",
-                           G_CALLBACK (category_focus_in), shell);
-          g_signal_connect (cc_shell_category_view_get_item_view (CC_SHELL_CATEGORY_VIEW (categoryview)),
-                           "focus-out-event",
-                           G_CALLBACK (category_focus_out), shell);
-          g_signal_connect (cc_shell_category_view_get_item_view (CC_SHELL_CATEGORY_VIEW (categoryview)),
-                           "keynav-failed",
-                           G_CALLBACK (keynav_failed), shell);
-
+          maybe_add_category_view (shell, dir_name);
 
           /* add the items from this category to the model */
           for (f = contents; f; f = f->next)
             {
               if (gmenu_tree_item_get_type (f->data) == GMENU_TREE_ITEM_ENTRY)
                 {
-                  cc_shell_model_add_item (CC_SHELL_MODEL (priv->store),
+                  cc_shell_model_add_item (CC_SHELL_MODEL (shell->priv->store),
                                            dir_name,
                                            f->data);
                 }
@@ -672,13 +636,43 @@ fill_model (GnomeControlCenter *shell)
     }
 
   g_slist_free (list);
+}
 
+static void
+on_menu_changed (GMenuTree          *monitor,
+                 GnomeControlCenter *shell)
+{
+  gtk_list_store_clear (shell->priv->store);
+  reload_menu (shell);
+}
+
+static void
+setup_model (GnomeControlCenter *shell)
+{
+  GnomeControlCenterPrivate *priv = shell->priv;
+
+  gtk_container_set_border_width (GTK_CONTAINER (shell->priv->main_vbox), 10);
+  gtk_container_set_focus_vadjustment (GTK_CONTAINER (shell->priv->main_vbox),
+                                       gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (shell->priv->scrolled_window)));
+
+  priv->store = (GtkListStore *) cc_shell_model_new ();
+  priv->category_views = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+  priv->menu_tree = gmenu_tree_lookup (MENUDIR "/gnomecc.menu", 0);
+
+  if (priv->menu_tree == NULL)
+    {
+      g_warning ("Could not find control center menu");
+      return;
+    }
+
+  reload_menu (shell);
+
+  gmenu_tree_add_monitor (priv->menu_tree, (GMenuTreeChangedFunc)on_menu_changed, shell);
 }
 
 static void
 load_panel_plugins (GnomeControlCenter *shell)
 {
-  GType panel_type;
   GList *modules;
 
   /* only allow this function to be run once to prevent modules being loaded
@@ -688,7 +682,7 @@ load_panel_plugins (GnomeControlCenter *shell)
     return;
 
   /* make sure the base type is registered */
-  panel_type = g_type_from_name ("CcPanel");
+  g_type_from_name ("CcPanel");
 
   shell->priv->extension_point
     = g_io_extension_point_register (CC_SHELL_PANEL_EXTENSION_POINT);
@@ -715,15 +709,16 @@ notebook_switch_page_cb (GtkNotebook               *book,
 {
   /* make sure the home button is shown on all pages except the overview page */
 
-  if (page_num == OVERVIEW_PAGE)
-    gtk_widget_hide (W (priv->builder, "home-button"));
+  if (page_num == OVERVIEW_PAGE || page_num == SEARCH_PAGE)
+    {
+      gtk_widget_hide (W (priv->builder, "home-button"));
+      gtk_widget_show (W (priv->builder, "search-entry"));
+    }
   else
-    gtk_widget_show (W (priv->builder, "home-button"));
-
-  if (page_num == CAPPLET_PAGE)
-    gtk_widget_hide (W (priv->builder, "search-entry"));
-  else
-    gtk_widget_show (W (priv->builder, "search-entry"));
+    {
+      gtk_widget_show (W (priv->builder, "home-button"));
+      gtk_widget_hide (W (priv->builder, "search-entry"));
+    }
 }
 
 /* CcShell implementation */
@@ -887,6 +882,17 @@ gnome_control_center_finalize (GObject *object)
       priv->default_window_icon = NULL;
     }
 
+  if (priv->menu_tree)
+    {
+      gmenu_tree_remove_monitor (priv->menu_tree, (GMenuTreeChangedFunc)on_menu_changed, object);
+      gmenu_tree_unref (priv->menu_tree);
+    }
+
+  if (priv->category_views)
+    {
+      g_hash_table_destroy (priv->category_views);
+    }
+
   G_OBJECT_CLASS (gnome_control_center_parent_class)->finalize (object);
 }
 
@@ -907,6 +913,16 @@ gnome_control_center_class_init (GnomeControlCenterClass *klass)
   shell_class->get_toplevel = _shell_get_toplevel;
 }
 
+static gboolean
+queue_resize (gpointer data)
+{
+  GtkWidget *widget = data;
+
+  gtk_widget_queue_resize (widget);
+
+  return FALSE;
+}
+
 static void
 on_window_size_allocate (GtkWidget          *widget,
                          GtkAllocation      *allocation,
@@ -939,7 +955,15 @@ on_window_size_allocate (GtkWidget          *widget,
       height = 50;
     }
 
-  gtk_scrolled_window_set_min_content_height (GTK_SCROLLED_WINDOW (self->priv->scrolled_window), height);
+  if (gtk_scrolled_window_get_min_content_height (GTK_SCROLLED_WINDOW (self->priv->scrolled_window)) != height)
+    {
+      gtk_scrolled_window_set_min_content_height (GTK_SCROLLED_WINDOW (self->priv->scrolled_window), height);
+      /*
+       * Queueing a resize out of size-allocate is ignored,
+       * so we have to defer to an idle.
+       */
+      g_idle_add (queue_resize, self->priv->scrolled_window);
+    }
 }
 
 static void
@@ -961,6 +985,35 @@ viewport_style_set_cb (GtkWidget *widget,
   g_signal_handlers_unblock_by_func (widget, viewport_style_set_cb, NULL);
 }
 
+static gboolean
+window_key_press_event (GtkWidget          *win,
+			GdkEventKey        *event,
+			GnomeControlCenter *self)
+{
+  gboolean retval;
+
+  retval = FALSE;
+
+  if (!gtk_widget_get_visible (self->priv->search_entry))
+    return retval;
+
+  if (event->state != 0 &&
+      (event->state & GDK_CONTROL_MASK))
+    {
+      switch (event->keyval)
+        {
+          case GDK_KEY_s:
+          case GDK_KEY_S:
+          case GDK_KEY_f:
+          case GDK_KEY_F:
+            gtk_widget_grab_focus (self->priv->search_entry);
+            retval = TRUE;
+            break;
+        }
+    }
+  return retval;
+}
+
 static void
 gnome_control_center_init (GnomeControlCenter *self)
 {
@@ -984,6 +1037,8 @@ gnome_control_center_init (GnomeControlCenter *self)
   /* connect various signals */
   priv->window = W (priv->builder, "main-window");
   g_signal_connect_swapped (priv->window, "delete-event", G_CALLBACK (g_object_unref), self);
+  g_signal_connect (priv->window, "key_press_event",
+                    G_CALLBACK (window_key_press_event), self);
 
   priv->notebook = W (priv->builder, "notebook");
   priv->scrolled_window = W (priv->builder, "scrolledwindow1");
@@ -1002,7 +1057,7 @@ gnome_control_center_init (GnomeControlCenter *self)
                     "clicked", G_CALLBACK (home_button_clicked_cb), self);
 
   /* load the available settings panels */
-  fill_model (self);
+  setup_model (self);
 
   /* load the panels that are implemented as plugins */
   load_panel_plugins (self);
